<?php

if (passive_authcache_builtin_serve_proxy()) {
  exit;
}

/**
 * Serves Authcache built-in backend as passive proxy.
 */
function passive_authcache_builtin_serve_proxy() {
  if (variable_get('passive_authcache_proxy_bypass_default')) {
    return FALSE;
  }

  if (passive_is_passive_proxy()) {
    // Serve passive response.
    if ($check = passive_check_passive_request()) {
      passive_authcache_builtin_serve_passive_response();
    }
    // Obtain the passive key for the current non-passive request.
    elseif (!isset($check) && $passive_key = passive_authcache_builtin_get_passive_key()) {
      return passive_serve_passive_prompt($passive_key);
    }
  }

  return FALSE;
}

/**
 * Checks backend response and alters Authcache built-in behavior.
 */
function passive_authcache_builtin_serve_passive_response() {
  // Force Authcache built-in backend to use Vary: X-Passive-Key.
  global $conf;
  drupal_add_http_header('Vary', 'X-Passive-Key');
  $conf['authcache_builtin_vary'] = 'X-Passive-Key';
  if (empty($conf['passive_request_key'])) {
    $conf['passive_request_key'] = 'passive_request_authcache_index';
  }
}

/**
 *
 */
function passive_request_authcache_index($request) {
  global $base_root;

  $url = $request['url'];
  $authcache_key = '';
  if (isset($request['headers']['cookie'])) {
    $session_id = passive_extract_session_id($request['headers']['cookie']);
    if ($cache = cache_get($base_root . ':' . $session_id, 'cache_authcache_key')) {
      $authcache_key = $cache->data;
    }
  }
  $hash_index = "url:$url;authcache_key:$authcache_key";
  return $hash_index;
}

/**
 * Obtains the Authcache key if the current request warrants a passive prompt.
 */
function passive_authcache_builtin_get_passive_key() {
  global $base_root;

  if (!function_exists('authcache_backend_init')) {
    return FALSE;
  }

  // Attempt to construct Authcache key.
  $key = FALSE;
  if (isset($_COOKIE[session_name()])) {
    $session_id = $_COOKIE[session_name()];
    // Retrieve cached Authcache key.
    // The key should be available upon login. We ignore all other scenarios
    // since the passive proxy is always responsible for passing back the given
    // passive key so even if one hasn't been generated yet the next request
    // will be different. (The risk of a security bypass is dependent on the
    // hash salt update and whether the reverse proxy is configured to defeat
    // any user attempt to smuggle a passive key through the proxy.)
    if ($cache = cache_get($base_root . ':' . $session_id, 'cache_authcache_key')) {
      $key = $cache->data;
    }
  }
  else {
    // Pass non-session requests through to regular page cache.
    return FALSE;
  }

  return $key;
}
